using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Datory;
using GxPress.Common.AppOptions;
using GxPress.Common.Tools;
using GxPress.Repository.Interface.SystemLabel;
using GxPress.Request.SystemLabel;
using Microsoft.Extensions.Options;

namespace GxPress.Repository.Implement.SystemLabel
{
    public class SystemLabelRepository : ISystemLabelRepository
    {
        private readonly Repository<Entity.SystemLabel.SystemLabel> _repository;
        private readonly IMapper _mapper;
        private readonly string _connectionString;
        private readonly string _databaseTypestr;
        public SystemLabelRepository(IOptionsMonitor<DatabaseOptions> dbOptionsAccessor, IMapper mapper)
        {
            _databaseTypestr = dbOptionsAccessor.CurrentValue.DatabaseType;
            _connectionString = dbOptionsAccessor.CurrentValue.ConnectionString;
            var databaseType =
                StringUtils.ToEnum<DatabaseType>(dbOptionsAccessor.CurrentValue.DatabaseType, DatabaseType.MySql);
            var database = new Database(databaseType, dbOptionsAccessor.CurrentValue.ConnectionString);
            _repository = new Repository<Entity.SystemLabel.SystemLabel>(database);
            _mapper = mapper;
        }

        public IDatabase Database => _repository.Database;
        public string TableName => _repository.TableName;
        public List<TableColumn> TableColumns => _repository.TableColumns;

        public async Task<Entity.SystemLabel.SystemLabel> GetAsync(int id)
        {
            return await _repository.GetAsync(id);
        }

        public async Task<bool> DeleteAsync(int id)
        {
            return await _repository.DeleteAsync(id);
        }

        public async Task<int> InsertAsync(Entity.SystemLabel.SystemLabel note)
        {
            return await _repository.InsertAsync(note);
        }

        public async Task<bool> UpdateAsync(SystemLabelUpRequest request)
        {
            if (request.Id > 0)
            {
                var model = await GetAsync(request.Id);
                if (request.IsDisable > 0)
                    model.IsDisable = request.IsDisable == 1;
                if (request.IsPage > 0)
                    model.IsPage = request.IsPage == 1;
                if (request.IsSkip > 0)
                    model.IsSkip = request.IsSkip == 1;
                if (!string.IsNullOrEmpty(request.LabelName))
                    model.LabelName = request.LabelName;
                if (!string.IsNullOrEmpty(request.LabelNameDescribe))
                    model.LabelNameDescribe = request.LabelNameDescribe;
                if (!string.IsNullOrEmpty(request.Remark))
                    model.Remark = request.Remark;
                if (request.ResourceType > 0)
                    model.ResourceType = request.ResourceType;
                if (request.Sort > 0)
                    model.Sort = request.Sort;
                if (!string.IsNullOrEmpty(request.StyleType))
                    model.StyleType = request.StyleType;
                return await _repository.UpdateAsync(model);
            }
            return false;
        }
        public async Task<bool> UpdateAsync(SqlKata.Query query)
        {
            return await _repository.UpdateAsync(query) > 0;
        }

        public async Task<IEnumerable<Entity.SystemLabel.SystemLabel>> GetAllAsync()
        {
            return await _repository.GetAllAsync(Q.Where(nameof(Entity.Navigations.Navigation.IsDisable), false).OrderByDesc(nameof(Entity.Navigations.Navigation.Sort)));
        }
    }
}