using GxPress.Service.Interface.Media;
using GxPress.Repository.Interface.Category;
using GxPress.Repository.Interface.Media;
using System.Threading.Tasks;
using GxPress.Result.Media;
using AutoMapper;
using System.Linq;
using GxPress.Result.Category;
using GxPress.Service.Interface.Epub;
using GxPress.Request.Media;

namespace GxPress.Service.Implement.Media
{
    public class MediaService : IMediaService
    {
        private readonly ICategoryRepository categoryRepository;
        private readonly IMediaRepository mediaRepository;
        private readonly IMapper _mapper;
        private readonly IEpubService epubService;
        private readonly IMediaLibraryRepository mediaLibraryRepository;
        public MediaService(ICategoryRepository categoryRepository, IMediaRepository mediaRepository, IMapper _mapper, IEpubService epubService, IMediaLibraryRepository mediaLibraryRepository)
        {
            this.categoryRepository = categoryRepository;
            this.mediaRepository = mediaRepository;
            this._mapper = _mapper;
            this.epubService = epubService;
            this.mediaLibraryRepository = mediaLibraryRepository;
        }
        public async Task<bool> InsertAsync(MediaResult result)
        {

            // if (result.CategoryId > 0)
            // {
            //     result.CategoryName = await categoryRepository.GetCategoryParentAsync(result.CategoryId, result.CategoryName);
            // }
            return await mediaRepository.InsertAsync(result);
        }
        public async Task<MediaResult> GetAsync(int id)
        {
            var result = new MediaResult();
            //获取媒体
            result = await mediaRepository.GetAsync(id);
            //获取类别
            var categoryAll = await categoryRepository.GetAllAsync();
            result.CategoryResults = categoryAll.Where(n => n.ParentId == 0).Select(n => _mapper.Map<CategoryResult>(n)).ToList();
            foreach (var item in result.CategoryResults)
            {
                item.IsChildren = categoryAll.Any(n => n.ParentId == item.Id);
                item.Children = categoryAll.Where(n => n.ParentId == item.Id).Select(n => _mapper.Map<CategoryResult>(n)).ToList();
                if (item.IsChildren)
                {
                    foreach (var chidren in item.Children)
                    {
                        chidren.IsChildren = categoryAll.Any(n => n.ParentId == chidren.Id);
                        chidren.Children = categoryAll.Where(n => n.ParentId == chidren.Id).Select(n => _mapper.Map<CategoryResult>(n)).ToList();
                    }
                }
            }

            return result;
        }
        /// <summary>
        /// 获取书籍详情
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<BookMediaResult> GetBookMediaResultAsync(int id)
        {
            var result = await mediaRepository.GetBookMediaResultAsync(id);
            if (result == null)
                throw new Common.Exceptions.BusinessException("不存在");
            if (result.MediaType != GxPress.EnumConst.ResourceTypeConst.Book.GetHashCode())
                throw new Common.Exceptions.BusinessException("非书籍");
            //获取书籍文件
            var mediaLibrary = await mediaLibraryRepository.GetTaskAsync(id);
            if (!mediaLibrary.FileUrl.Contains("epub"))
                throw new Common.Exceptions.BusinessException("书籍不存在epub文件");
            //书籍
            result.BookCatalogResults = epubService.GetCatalog("wwwroot" + mediaLibrary.FileUrl);
            //计算免费占比
            return result;
        }
        /// <summary>
        /// 获取视频详情
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<VideoMediaResult> GetVideoMediaResultAsync(int id)
        {
            var result = await mediaRepository.GetVideoMediaResultAsync(id);
            if (result == null)
                throw new Common.Exceptions.BusinessException("不存在");
            if (result.MediaType != GxPress.EnumConst.ResourceTypeConst.Video.GetHashCode())
                throw new Common.Exceptions.BusinessException("非视频");
            //获取视频
            result.MediaLibraryResults = await mediaLibraryRepository.GetAllAsync(id);

            return result;
        }
        /// <summary>
        /// 获取章节内容
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> GetBookMediaContentResultAsync(BookCatalogRequest request)
        {
            if (request.MediaId == 0)
                throw new Common.Exceptions.BusinessException("请检查参数");
            if (string.IsNullOrEmpty(request.CatalogNameg))
                throw new Common.Exceptions.BusinessException("请检查参数");
            if (string.IsNullOrEmpty(request.CatalogId))
                throw new Common.Exceptions.BusinessException("请检查参数");
            //获取书籍文件
            var mediaLibrary = await mediaLibraryRepository.GetTaskAsync(request.MediaId);
            if (!mediaLibrary.FileUrl.Contains("epub"))
                throw new Common.Exceptions.BusinessException("书籍不存在epub文件");
            request.Path = "wwwroot" + mediaLibrary.FileUrl;
            return epubService.GetBookCatalogContent(request);
        }


    }
}